@@ -12,7 +12,6 @@
 package Lemonldap::NG::Manager::Notifications;
 
 use strict;
-use File::Basename;
 use Lemonldap::NG::Handler::CGI qw(:globalStorage :locationRules);
 use Lemonldap::NG::Portal::Notification;
 use Lemonldap::NG::Common::Conf;              #link protected conf Configuration
@@ -107,10 +106,6 @@ sub new {
     # Multi values separator
     $self->{multiValuesSeparator} ||= '; ';
 
-    # Absolute path to the htdocs directory where is manager script.
-    my ( $mname, $mpath, $msuffix ) = fileparse( $ENV{SCRIPT_FILENAME} );
-    $self->{managerHtdocsPath} = $mpath;
-
     # Now we're ready to display sessions. Choose display type
     foreach my $k ( $self->param() ) {
 
@@ -639,7 +634,7 @@ notifications
   our $cgi ||= Lemonldap::NG::Manager::Notifications->new({
         localStorage        => "Cache::FileCache",
         localStorageOptions => {
-            'namespace'          => 'MyNamespace',
+            'namespace'          => 'lemonldap-ng',
             'default_expires_in' => 600,
             'directory_umask'    => '007',
             'cache_root'         => '/tmp',
