@@ -9,7 +9,6 @@ use strict;
 use XML::LibXML;
 use XML::LibXSLT;
 use MIME::Base64;
-use JSON;
 use LWP::Simple;
 use LWP::UserAgent;
 
@@ -18,9 +17,10 @@ use Lemonldap::NG::Common::Safelib;        #link protected safe Safe object
 use Lemonldap::NG::Manager::Downloader;    #inherits
 use Lemonldap::NG::Manager::_Struct;       #link protected struct _Struct object
 use Lemonldap::NG::Manager::_i18n;
+use Lemonldap::NG::Manager::Request;
 use Lemonldap::NG::Common::Conf::Constants;    #inherits
 
-our $VERSION = '0.992';
+our $VERSION = '1.2.0';
 our ( $stylesheet, $parser );
 
 ## @method void confUpload(ref rdata)
@@ -86,7 +86,7 @@ sub confUpload {
 
         my $NK = 0;
         $id =~
-          s/^text_(NewID_)?li_(\w+)(\d)(?:_\d+)?$/decode_base64($2.'='x $3)/e;
+s/^text_(NewID_)?li_([\w\/\+\=]+)(\d)(?:_\d+)?$/decode_base64($2.'='x $3)/e;
         $NK = 1 if ($1);
         $id =~ s/\r//g;
         $id =~ s/^\///;
@@ -146,7 +146,7 @@ sub confUpload {
 
             # Special case: avoid bug with node created from parent node
             if ( $id =~
-/^(virtualHosts|samlIDPMetaDataExportedAttributes|samlSPMetaDataExportedAttributes|generalParameters\/authParams\/choiceParams)/
+/^(virtualHosts|samlIDPMetaDataNode|samlSPMetaDataNode|generalParameters\/authParams\/choiceParams)/
               )
             {
                 $self->lmLog( "Special trigger for $id (attribute $name)",
@@ -165,11 +165,11 @@ s/^virtualHosts\/([^\/]*)?\/header.*/exportedHeaders\/$1\/$name/;
 
                 # SAML IDP attribute
                 $id =~
-s/^samlIDPMetaDataExportedAttributes\/([^\/]*)?.*/samlIDPMetaDataExportedAttributes\/$1\/$name/;
+s/^samlIDPMetaDataNode\/([^\/]*)?.*/samlIDPMetaDataExportedAttributes\/$1\/$name/;
 
                 # SAML SP attribute
                 $id =~
-s/^samlSPMetaDataExportedAttributes\/([^\/]*)?.*/samlSPMetaDataExportedAttributes\/$1\/$name/;
+s/^samlSPMetaDataNode\/([^\/]*)?.*/samlSPMetaDataExportedAttributes\/$1\/$name/;
 
                 # Authentication choice
                 $id =~
@@ -356,7 +356,12 @@ s/^(samlSPMetaDataXML|samlSPMetaDataExportedAttributes|samlSPMetaDataOptions)\/(
         }
     }
 
-    # 1.5 Global tests
+    # 1.5 Author attributes for accounting
+    $newConf->{cfgAuthor}   = $ENV{REMOTE_USER} || 'anonymous';
+    $newConf->{cfgAuthorIP} = $ENV{REMOTE_ADDR};
+    $newConf->{cfgDate}     = time();
+
+    # 1.6 Global tests
     $self->lmLog( "Launch global tests", 'debug' );
     {
         my $tests = $self->globalTests($newConf);
@@ -364,24 +369,23 @@ s/^(samlSPMetaDataXML|samlSPMetaDataExportedAttributes|samlSPMetaDataOptions)\/(
             my ( $res, $msg );
             eval {
                 ( $res, $msg ) = $sub->();
-                if ($res) {
+                if ( $res == -1 ) {
+                    $errors->{force}->{$name} = $msg
+                      unless ( $self->param('force') );
+                }
+                elsif ($res) {
                     if ($msg) {
                         $errors->{warnings}->{$name} = $msg;
                     }
                 }
                 else {
-                    $errors->{error}->{$name} = $msg;
+                    $errors->{errors}->{$name} = $msg;
                 }
             };
             $errors->{warnings}->{$name} = "Test $name failed: $@" if ($@);
         }
     }
 
-    # 1.6 Author attributes for accounting
-    $newConf->{cfgAuthor}   = $ENV{REMOTE_USER} || 'anonymous';
-    $newConf->{cfgAuthorIP} = $ENV{REMOTE_ADDR};
-    $newConf->{cfgDate}     = time();
-
     # 2. SAVE CONFIGURATION
 
     $errors->{result}->{other} = '';
@@ -393,6 +397,19 @@ s/^(samlSPMetaDataXML|samlSPMetaDataExportedAttributes|samlSPMetaDataOptions)\/(
         $self->_sub( 'userInfo',
             "Configuration rejected for $newConf->{cfgAuthor}: syntax error" );
     }
+    elsif ( $errors->{force} ) {
+        $errors->{result}->{cfgNum} = 0;
+        $errors->{result}->{msg}    = $self->translate('warning');
+        $self->_sub( 'userInfo',
+"Configuration rejected for $newConf->{cfgAuthor}: confirmation needed"
+        );
+        $errors->{result}->{other} = '* <a href=\'javascript:uploadConf(1)\'>'
+          . $self->translate('clickHereToForce') . '</a>';
+        foreach my $k ( keys %{ $errors->{force} } ) {
+            $errors->{errors}->{$k} =
+              delete( $errors->{force}->{$k} ) . '<sup>*</sup>';
+        }
+    }
 
     # 2.2 Try to save configuration
     else {
@@ -440,11 +457,13 @@ s/^(samlSPMetaDataXML|samlSPMetaDataExportedAttributes|samlSPMetaDataOptions)\/(
                 UPLOAD_DENIED,      'uploadDenied',
                 SYNTAX_ERROR,       'syntaxError',
                 DEPRECATED,         'confModuledeprecated',
-            }->{ $errors->{result}->{cfgNum} };
+              }->{ $errors->{result}->{cfgNum} }
+              || 'unknownError';
 
             # Log failure using Lemonldap::NG::Common::CGI::userError()
             $self->_sub( 'userError',
-                "Configuration rejected for $newConf->{cfgAuthor}: $msg" );
+                "Configuration rejected for $newConf->{cfgAuthor}: "
+                  . $Lemonldap::NG::Common::Conf::msg );
         }
 
         # Translate msg returned
@@ -452,7 +471,7 @@ s/^(samlSPMetaDataXML|samlSPMetaDataExportedAttributes|samlSPMetaDataOptions)\/(
         if (   $errors->{result}->{cfgNum} == CONFIG_WAS_CHANGED
             or $errors->{result}->{cfgNum} == DATABASE_LOCKED )
         {
-            $errors->{result}->{other} = '<a href="javascript:uploadConf(1)">'
+            $errors->{result}->{other} = '<a href=\'javascript:uploadConf(1)\'>'
               . $self->translate('clickHereToForce') . '</a>';
         }
         elsif ( $errors->{result}->{cfgNum} == DEPRECATED ) {
@@ -461,33 +480,11 @@ s/^(samlSPMetaDataXML|samlSPMetaDataExportedAttributes|samlSPMetaDataOptions)\/(
     }
 
     # 3. PREPARE JSON RESPONSE
-    my $buf = '{';
-    my $i   = 0;
-    while ( my ( $type, $h ) = each %$errors ) {
-        $buf .= ',' if ($i);
-        $buf .= "\"$type\":{";
-        $buf .= join(
-            ',',
-            map {
-                $h->{$_} =~ s/"/\\"/g;
-                $h->{$_} =~ s/\n/ /g;
-                "\"$_\":\"$h->{$_}\""
-              } keys %$h
-        );
-        $buf .= '}';
-        $i++;
-    }
-    $buf .= '}';
+    binmode( STDOUT, ':bytes' );
 
     # 4. SEND JSON RESPONSE
-    binmode( STDOUT, ':bytes' );    # Else JSON is invalid
-    use utf8;
-    utf8::encode($buf);             # Reencode message
-    print $self->header(
-        -type           => 'application/json; charset=utf-8',
-        -Content_Length => length($buf)
-    );
-    print $buf;
+    $self->sendJSONResponse($errors);
+
     $self->quit();
 }
 
@@ -522,26 +519,7 @@ sub fileUpload {
         $content =~ s!<!&lt;!g;
         $content =~ s!>!&gt;!g;
 
-        # Red Hat / Debian compatibilities
-        my $json         = new JSON();
-        my $json_content = '';
-        if ( $JSON::VERSION lt 2 ) {
-            local $JSON::UTF8 = 1;
-            $json_content = $json->objToJson( [$content] );
-            $json_content =~ s/^\[//;
-            $json_content =~ s/\]$//;
-        }
-        else {
-            $json = $json->allow_nonref( ['1'] );
-            $json = $json->utf8(         ['1'] );
-            $json_content = $json->encode($content);
-        }
-
-        my $content = '{"status":"OK", "content":' . $json_content . '}';
-        print $self->header(
-            -type           => 'text/html; charset=utf-8',
-            -Content_Length => length $content
-        ) . $content;
+        $self->sendJSONResponse($content);
     }
 
     $self->quit();
@@ -566,27 +544,9 @@ sub urlUpload {
     $content =~ s!<!&lt;!g;
     $content =~ s!>!&gt;!g;
 
-    # Build JSON reponse
-    # Require Red Hat and Debian compatibilites
-    my $json         = new JSON();
-    my $json_content = '';
-    if ( $JSON::VERSION lt 2 ) {
-        local $JSON::UTF8 = 1;
-        $json_content = $json->objToJson( [$content] );
-        $json_content =~ s/^\[//;
-        $json_content =~ s/\]$//;
-    }
-    else {
-        $json = $json->allow_nonref( ['1'] );
-        $json = $json->utf8(         ['1'] );
-        $json_content = $json->encode($content);
-    }
+    $self->sendJSONResponse($content);
 
-    $content = '{"status":"OK", "content":' . $json_content . '}';
-    print $self->header(
-        -type           => 'text/html; charset=utf-8',
-        -Content_Length => length $content
-    ) . $content;
+    $self->quit();
 }
 
 ## @method protected array applyTest(void* test,string value)
@@ -703,7 +663,7 @@ sub setKeyToH {
                 unless ( ref( $tmp->{$key} ) ) {
                     $self->lmLog(
 "setKeyToH: k2 $k2 set, but $key is not a reference, create it",
-                        'error'
+                        'debug'
                     );
                     $tmp->{$key} = {};
                 }
@@ -782,15 +742,18 @@ sub applyConf {
     my $status;
 
     # Get apply section values
-    my $localConf = $self->confObj->getLocalConf( APPLYSECTION, undef, 0 );
+    my %reloadUrls =
+      %{ $self->confObj->getLocalConf( APPLYSECTION, undef, 0 ) };
+    %reloadUrls = %{ $self->confObj->getConf->{reloadUrls} }
+      unless (%reloadUrls);
 
     # Create user agent
     my $ua = new LWP::UserAgent( requests_redirectable => [] );
     $ua->timeout(10);
 
     # Parse apply values
-    foreach ( keys %$localConf ) {
-        my ( $host, $request ) = ( $_, $localConf->{$_} );
+    foreach ( sort keys %reloadUrls ) {
+        my ( $host, $request ) = ( $_, $reloadUrls{$_} );
         my ( $method, $vhost, $uri ) =
           ( $request =~ /^(https?):\/\/([^\/]+)(.*)$/ );
         unless ($vhost) {
